/*

=== wunderDB JavaScript SDK 
=== Version: 1.1.0
=== Author: Tanmoy Sen Gupta

Legends -
    [-] - Yet to be Implemented
    [/] - Implemented

Methods in SDK 
[/] validate-connection
[/] initialize
[-] get-cluster
[-] get-database
[/] create-database
[/] delete-database
[/] create-collection
[/] clone-collection
[/] migrate-collection
[-] delete-collection
[-] get-collection
[-] add-data
[-] update-data
[-] delete-data
[-] get-data

*/

// Custom State Hook 
// Source : https://codesandbox.io/s/ff4rx?file=/src/index.js:0-628

function useState(defaultValue) {
    let value = defaultValue;
    function getValue() { return value; }
    function setValue(newValue) { value = newValue; }
    return [getValue, setValue];
}

// Connection Methods

function validate_connection(cluster_id, token) {
    const [response, setResponse] = useState()
    if (cluster_id && token) {
        fetch(`https://wdb.tanmoysg.com/validate-connection?cluster=${cluster_id}&token=${token}`, {
            method: "GET",
            cache: "no-cache",
            headers: { "content_type": "application/json" }
        }).then(response => {
            return response.json()
        }).then(json => {
            if (json.status_code === '1') {
                setResponse(true);
            } else {
                setResponse(false);
            }
        })
    }
    return response;
}

function initialize(cluster_id, token) {
    const [response, setResponse] = useState();
    if (validate_connection(cluster_id, token)) {
        setResponse(`wdb.tanmoysg.com/connect?cluster=${cluster_id}&token=${token}`);
        return response;
    } else {
        throw 'wrong_cluster_or_token'
    }
}



// Database Methods

function create_database(cluster, name) {
    const [response, setResponse] = useState()
    if (cluster && name) {
        fetch(cluster, {
            method: "POST",
            cache: "no-cache",
            headers: { "content_type": "application/json" },
            body: {
                "action": "create-database",
                "payload": {
                    "name": name
                }
            }
        }).then(response => {
            return response.json()
        }).then(json => {
            setResponse(json.response);
        })
    } else {
        throw 'wrong_request_format';
    }
    return response;

}

function delete_database(cluster, targetDatabase, migrate = 'false', destinationDatabase, ifCollectionExists = 'skip') {
    const [response, setResponse] = useState();

    if (cluster && targetDatabase && destinationDatabase) {
        fetch(cluster, {
            method: "POST",
            cache: "no-cache",
            headers: { "content_type": "application/json" },
            body: {
                "action": "delete-database",
                "payload": {
                    "targetDatabase": targetDatabase,
                    "migrateCollection": migrate,
                    "destinationDatabase": destinationDatabase,
                    "ifCollectionExists": ifCollectionExists
                }
            }
        }).then(response => {
            return response.json()
        }).then(json => {
            setResponse(json);
        })
    } else {
        throw 'wrong_request_format';
    }
    return response
}

// Collection Methods

function create_collection(cluster, database, name, schema) {
    const [response, setResponse] = useState();

    if (cluster && database && name && schema) {
        fetch(cluster, {
            method: "POST",
            cache: "no-cache",
            headers: { "content_type": "application/json" },
            body: {
                "action": "create-collection",
                "payload": {
                    "database": database,
                    "name": name,
                    "schema": schema
                }
            }
        }).then(response => {
            return response.json()
        }).then(json => {
            if (json.status_code === '1') {
                setResponse(json.response);
            } else {
                setResponse(json.response);
            }
        })
    } else {
        throw 'wrong_request_format';
    }
    return response;
}

function clone_collection(cluster, sourceDatabase, collection, destinationDatabase, actionIfExists = 'abort', newName) {
    const [response, setResponse] = useState();

    if (cluster && sourceDatabase && collection && destinationDatabase) {
        fetch(cluster, {
            method: "POST",
            cache: "no-cache",
            headers: { "content_type": "application/json" },
            body: {
                "action": "clone-collection",
                "payload": {
                    "sourceDatabase": sourceDatabase,
                    "collection": collection,
                    "destinationDatabase": destinationDatabase,
                    "actionIfExists": actionIfExists,
                    "newName": newName
                }
            }
        }).then(response => {
            return response.json()
        }).then(json => {
            if (json.status_code === '1') {
                setResponse(json.response)
            } else {
                setResponse(json.response)
            }
        })
    } else {
        throw 'wrong_request_format';
    }
    return response
}

function migrate_collection(cluster, sourceDatabase, collection, destinationDatabase, actionIfExists = 'abort', newName) {
    const [response, setResponse] = useState();

    if (cluster && sourceDatabase && collection && destinationDatabase) {
        fetch(cluster, {
            method: "POST",
            cache: "no-cache",
            headers: { "content_type": "application/json" },
            body: {
                "action": "migrate-collection",
                "payload": {
                    "sourceDatabase": sourceDatabase,
                    "collection": collection,
                    "destinationDatabase": destinationDatabase,
                    "actionIfExists": actionIfExists,
                    "newName": newName
                }
            }
        }).then(response => {
            return response.json()
        }).then(json => {
            if (json.status_code === '1') {
                setResponse(json.response)
            } else {
                setResponse(json.response)
            }
        })
    } else {
        throw 'wrong_request_format';
    }
    return response;
}

function delete_collection(cluster, database, collection) {
    const [response, setResponse] = useState();

    if (cluster && database && collection) {
        fetch(cluster, {
            method: "POST",
            cache: "no-cache",
            headers: { "content_type": "application/json" },
            body: {
                "action": "delete-collection",
                "payload": {
                    "database": database,
                    "collection": collection
                }
            }
        }).then(response => {
            return response.json()
        }).then(json => {
            if (json.status_code === '1') {
                setResponse(json.response)
            } else {
                setResponse(json.response)
            }
        })
    } else {
        throw 'wrong_request_format';
    }
    return response;
}

export default {
    validate_connection,
    initialize,
    create_database,
    delete_database,
    create_collection,
    clone_collection,
    migrate_collection,
    delete_collection
};

